#ifndef _TLINKPOOL_H
#define _TLINKPOOL_H

#include "TlinkType.h"

#define NUM_LINK_POOLS 	   100
		// how many pools in total

#define NUM_LINKS_IN_POOL  20000
		// how many links in each pool

//
//
// class: TlinkPool
//
//		  Acts as an memory manager for TlinkType
//		  Support functionality:
//			1. Batch allocation, whenevern necessary NUM_LINKS_IN_POOL
//			     number of TlinkType is allocated, instead of allocating
//			     one link at a time
//			2. Supports freeing of all existing allocated links and reusing
//			     the allocated memory subsequently
//
//
class TlinkPool {

	private:
		TlinkType		*fLinkPool[NUM_LINK_POOLS];
							// all the pools;

		int				fAllocatedPools;
							//  total number of allocated pools

		int				fCurrentPoolIndex;
							// which pool is currently active

		int				fCurrentLinkInPoolIndex;
							// which link in a pool is still free

	public:
		TlinkPool();
			// initialize pool status _WITHOUT_ 
			// allocating any memory

			// memory is only free when 
			// the destructor is called
		~TlinkPool() {
			// fprintf(stderr, "Freeing TlinkPool: Total Allocated Pools(%d)\n", fAllocatedPools);
		}

		TlinkType*		nextFreeLink();
							// next free (available) link

		void			resetAllLinks();
							// reset all indices _WITHOUT_
							// freeing any memory, so that
							// we can reuse the already allcoated
							// memory
};

#endif
